/***************************************************************************
 *   This file is part of Dimmable Light for Arduino, a library to         *
 *   control dimmers.                                                      *
 *                                                                         *
 *   Copyright (C) 2018-2022  Fabiano Riccardi                             *
 *                                                                         *
 *   Dimmable Light for Arduino is free software; you can redistribute     *
 *   it and/or modify it under the terms of the GNU Lesser General Public  *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, see <http://www.gnu.org/licenses/>   *
 ***************************************************************************/
#ifndef HW_TIMER_ESP32_H
#define HW_TIMER_ESP32_H

#include <Arduino.h>

typedef struct {
  union {
    struct {
      uint32_t reserved0 : 10;
      uint32_t alarm_en : 1;     /*When set  alarm is enabled*/
      uint32_t level_int_en : 1; /*When set  level type interrupt will be generated during alarm*/
      uint32_t edge_int_en : 1;  /*When set  edge type interrupt will be generated during alarm*/
      uint32_t divider : 16;     /*Timer clock (T0/1_clk) pre-scale value.*/
      uint32_t autoreload : 1;   /*When set  timer 0/1 auto-reload at alarming is enabled*/
      uint32_t increase : 1; /*When set  timer 0/1 time-base counter increment. When cleared timer 0
                                time-base counter decrement.*/
      uint32_t enable : 1;   /*When set  timer 0/1 time-base counter is enabled*/
    };
    uint32_t val;
  } config;
  uint32_t cnt_low;  /*Register to store timer 0/1 time-base counter current value lower 32 bits.*/
  uint32_t cnt_high; /*Register to store timer 0 time-base counter current value higher 32 bits.*/
  uint32_t update; /*Write any value will trigger a timer 0 time-base counter value update (timer 0
                      current value will be stored in registers above)*/
  uint32_t alarm_low;  /*Timer 0 time-base counter value lower 32 bits that will trigger the alarm*/
  uint32_t alarm_high; /*Timer 0 time-base counter value higher 32 bits that will trigger the
                          alarm*/
  uint32_t load_low;   /*Lower 32 bits of the value that will load into timer 0 time-base counter*/
  uint32_t load_high;  /*higher 32 bits of the value that will load into timer 0 time-base counter*/
  uint32_t reload;     /*Write any value will trigger timer 0 time-base counter reload*/
} hw_timer_reg_t;

typedef struct hw_timer_s {
  hw_timer_reg_t* dev;
  uint8_t num;
  uint8_t group;
  uint8_t timer;
  portMUX_TYPE lock;
} hw_timer_t;

void timerInit(void (*callback)());

void setCallback(void (*callback)());

void startTimerAndTrigger(uint32_t delay);

void stopTimer();

#endif  // END HW_TIMER_ESP32_H